#include "Rivet/Analysis.hh"
#include "Rivet/Tools/Logging.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/FastJets.hh"
#include "Rivet/Projections/ChargedFinalState.hh"
#include "Rivet/Projections/InvMassFinalState.hh"
#include "Rivet/Tools/ParticleIdUtils.hh"
#include "Rivet/Math/Vector4.hh"
#include "Rivet/Projections/ZFinder.hh"
#include "Rivet/Projections/VisibleFinalState.hh"
#include "Rivet/Projections/VetoedFinalState.hh"

namespace Rivet
{

  class CMS_2015_I1310737:public Analysis
  {
  public:

    /// Constructor
    CMS_2015_I1310737():Analysis("CMS_2015_I1310737")
    {
    }

  public:

    /// Book histograms and initialise projections before the run
    void init()
    {

      // set optionally the verbosity for the internal Rivet message system
      //      getLog().setLevel(0);


      ZFinder ZeeFinder(FinalState(), Cuts::abseta < 2.4 && Cuts::pT > 20*GeV, PID::ELECTRON, 71.0*GeV, 111.0*GeV);
      addProjection(ZeeFinder, "ZeeFinder");

      ZFinder ZmumuFinder(FinalState(), Cuts::abseta < 2.4 && Cuts::pT > 20*GeV, PID::MUON, 71.0*GeV, 111.0*GeV);
      addProjection(ZmumuFinder, "ZmumuFinder");

      VetoedFinalState jetConstits = VetoedFinalState(VisibleFinalState());
      jetConstits.addVetoOnThisFinalState(ZeeFinder);
      jetConstits.addVetoOnThisFinalState(ZmumuFinder);

      FastJets akt05Jets = FastJets(jetConstits, FastJets::ANTIKT, 0.5);
      addProjection(akt05Jets, "AntiKt05Jets");

      _h_excmult_jets_tot = bookHisto1D(1, 1, 1);
      _h_incmult_jets_tot = bookHisto1D(2, 1, 1);
      _h_leading_jet_pt_tot = bookHisto1D(3, 1, 1);
      _h_second_jet_pt_tot = bookHisto1D(4, 1, 1);
      _h_third_jet_pt_tot = bookHisto1D(5, 1, 1);
      _h_fourth_jet_pt_tot = bookHisto1D(6, 1, 1);
      _h_leading_jet_eta_tot = bookHisto1D(7, 1, 1);
      _h_second_jet_eta_tot = bookHisto1D(8, 1, 1);
      _h_third_jet_eta_tot = bookHisto1D(9, 1, 1);
      _h_fourth_jet_eta_tot = bookHisto1D(10, 1, 1);
      _h_ht1_tot = bookHisto1D(11, 1, 1);
      _h_ht2_tot = bookHisto1D(12, 1, 1);
      _h_ht3_tot = bookHisto1D(13, 1, 1);
      _h_ht4_tot = bookHisto1D(14, 1, 1);

    }

    /// Perform the per-event analysis
    void analyze(const Event & event)
    {

      const double weight = event.weight();

      const ZFinder& ZeeFS = applyProjection<ZFinder>(event, "ZeeFinder");
      const ZFinder& ZmumuFS = applyProjection<ZFinder>(event, "ZmumuFinder");

      const Particles& Zees = ZeeFS.bosons();
      const Particles& Zmumus = ZmumuFS.bosons();

      // we did not find exactly one Z. no good.
      if (Zees.size() + Zmumus.size() != 1) {
          MSG_DEBUG("did not find exactly one good Z candidate; veto.");
          vetoEvent;
      }


      // find the (dressed!) leptons
      const Particles& dressedLeptons = Zees.size() ? ZeeFS.constituents() : ZmumuFS.constituents();

      // cluster jets
      // veto has already been applied on leptons and photons used for
      // dressing.
      const Jets& jets = applyProjection<FastJets>(event, "AntiKt05Jets").jetsByPt(Cuts::abseta < 2.4 && Cuts::pT > 30*GeV);

      // perform lepton-jet overlap and ht calculation
      double ht = 0;
      Jets goodjets;
      foreach (const Jet& j, jets) {
          bool overlap = false;
          foreach (const Particle& l, dressedLeptons) {
              if (Rivet::deltaR(j, l) < 0.5) {
                  overlap = true;
                  break;
              }
          }

          if (!overlap) {
              goodjets.push_back(j);
              ht += j.pT();
          }
      }


      // we don't care about events with no jets.
      if (goodjets.size() == 0) {
          MSG_DEBUG("no jets in event; veto.");
          vetoEvent;
      }


      for (unsigned int iJet = 1; iJet <= goodjets.size(); iJet++ )
        _h_incmult_jets_tot->fill(iJet, weight*0.5);

      // fill leading jet histograms
      Jet& jet = goodjets.at(0);

      _h_leading_jet_pt_tot->fill(jet.pT(), weight*0.5);
      _h_leading_jet_eta_tot->fill(jet.abseta(), weight*0.5);
      _h_ht1_tot->fill(ht, weight*0.5);
      _h_excmult_jets_tot->fill(goodjets.size(), weight*0.5);

      if (goodjets.size() > 1) {
          jet = goodjets.at(1);
          _h_second_jet_pt_tot->fill(jet.pT(), weight*0.5);
          _h_second_jet_eta_tot->fill(jet.abseta(), weight*0.5);
          _h_ht2_tot->fill(ht, weight*0.5);
      }

      if (goodjets.size() > 2) {
          jet = goodjets.at(2);
          _h_third_jet_pt_tot->fill(jet.pT(), weight*0.5);
          _h_third_jet_eta_tot->fill(jet.abseta(), weight*0.5);
          _h_ht3_tot->fill(ht, weight*0.5);
      }

      if (goodjets.size() > 3) {
          jet = goodjets.at(3);
          _h_fourth_jet_pt_tot->fill(jet.pT(), weight*0.5);
          _h_fourth_jet_eta_tot->fill(jet.abseta(), weight*0.5);
          _h_ht4_tot->fill(ht, weight*0.5);
      }

      return;
    }

    /// Normalise histograms etc., after the run
    void finalize()
    {
      
      double norm(1.);
      if ( std::fabs(sumOfWeights()) > 0. ) norm = crossSection()/sumOfWeights();

      MSG_INFO("Cross section = " << std::setfill(' ') << std::setw(14) << std::fixed << std::setprecision(3) << crossSection() << " pb");
      MSG_INFO("# Events      = " << std::setfill(' ') << std::setw(14) << std::fixed << std::setprecision(3) << numEvents() );
      MSG_INFO("SumW          = " << std::setfill(' ') << std::setw(14) << std::fixed << std::setprecision(3) << sumOfWeights());
      MSG_INFO("Norm factor   = " << std::setfill(' ') << std::setw(14) << std::fixed << std::setprecision(6) << norm);
      
      scale(_h_excmult_jets_tot, norm );
      scale(_h_incmult_jets_tot, norm );
      scale(_h_leading_jet_pt_tot, norm );
      scale(_h_second_jet_pt_tot, norm );
      scale(_h_third_jet_pt_tot, norm );
      scale(_h_fourth_jet_pt_tot, norm );
      scale(_h_leading_jet_eta_tot, norm );
      scale(_h_second_jet_eta_tot, norm );
      scale(_h_third_jet_eta_tot, norm );
      scale(_h_fourth_jet_eta_tot, norm );
      scale(_h_ht1_tot, norm );
      scale(_h_ht2_tot, norm );
      scale(_h_ht3_tot, norm );
      scale(_h_ht4_tot, norm );


    }

  private:

    // Data members like post-cuts event weight counters go here

  private:

    /// @name Histograms

    Histo1DPtr _h_excmult_jets_tot;
    Histo1DPtr _h_incmult_jets_tot;
    Histo1DPtr _h_leading_jet_pt_tot;
    Histo1DPtr _h_second_jet_pt_tot;
    Histo1DPtr _h_third_jet_pt_tot;
    Histo1DPtr _h_fourth_jet_pt_tot;
    Histo1DPtr _h_leading_jet_eta_tot;
    Histo1DPtr _h_second_jet_eta_tot;
    Histo1DPtr _h_third_jet_eta_tot;
    Histo1DPtr _h_fourth_jet_eta_tot;
    Histo1DPtr _h_ht1_tot;
    Histo1DPtr _h_ht2_tot;
    Histo1DPtr _h_ht3_tot;
    Histo1DPtr _h_ht4_tot;

  };

  // This global object acts as a hook for the plugin system
  AnalysisBuilder<CMS_2015_I1310737> plugin_CMS_2015_I1310737;

}
